@@ -36,7 +36,11 @@ uiflags = [
   -- flagNone ["debug-ui"] (setboolopt "rules-file") "run with no terminal output, showing console"
    flagNone ["watch","w"] (setboolopt "watch") "watch for data and date changes and reload automatically"
   ,flagReq  ["theme"] (\s opts -> Right $ setopt "theme" s opts) "THEME" ("use this custom display theme ("++intercalate ", " themeNames++")")
-  ,flagReq  ["register"] (\s opts -> Right $ setopt "register" s opts) "ACCTREGEX" "start in the (first) matched account's register"
+  ,flagNone ["cash"] (setboolopt "cash") "start in the cash accounts screen"
+  ,flagNone ["bs"] (setboolopt "bs") "start in the balance sheet accounts screen"
+  ,flagNone ["is"] (setboolopt "is") "start in the income statement accounts screen"
+  ,flagNone ["all"] (setboolopt "all") "start in the all accounts screen"
+  ,flagReq  ["register"] (\s opts -> Right $ setopt "register" s opts) "ACCTREGEX" "start in the (first matched) account's register"
   ,flagNone ["change"] (setboolopt "change")
     "show period balances (changes) at startup instead of historical balances"
   -- ,flagNone ["cumulative"] (setboolopt "cumulative")
@@ -52,16 +56,19 @@ uiflags = [
 
 --uimode :: Mode RawOpts
 uimode =  (mode "hledger-ui" (setopt "command" "ui" def)
-            "browse accounts, postings and entries in a full-window curses interface"
-            (argsFlag "[PATTERNS]") []){
+            "browse accounts, postings and entries in a full-window TUI"
+            (argsFlag "[--cash|--bs|--is|--all|--register=ACCT] [QUERY]") []){
               modeGroupFlags = Group {
                                 groupUnnamed = uiflags
                                ,groupHidden = hiddenflags
-                                 ++ [flagNone ["future"] (setboolopt "forecast") "compatibility alias, use --forecast instead"]
+                                 ++
+                                 [flagNone ["future"] (setboolopt "forecast") "old flag, use --forecast instead"
+                                 ,flagNone ["menu"] (setboolopt "menu") "old flag, menu screen is now the default"
+                                 ]
                                ,groupNamed = [(generalflagsgroup1)]
                                }
              ,modeHelpSuffix=[
-                  -- "Reads your ~/.hledger.journal file, or another specified by $LEDGER_FILE or -f, and starts the full-window curses ui."
+                  -- "Reads your ~/.hledger.journal file, or another specified by $LEDGER_FILE or -f, and starts the full-window TUI."
                  ]
            }
 
@@ -101,7 +108,7 @@ getHledgerUIOpts :: IO UIOpts
 --getHledgerUIOpts = processArgs uimode >>= return >>= rawOptsToUIOpts
 getHledgerUIOpts = do
   args <- getArgs >>= expandArgsAt
-  let args' = replaceNumericFlags args
+  let args' = replaceNumericFlags $ ensureDebugHasArg args
   let cmdargopts = either usageError id $ process uimode args'
   rawOptsToUIOpts cmdargopts
 
